@@ -3,6 +3,7 @@ from __future__ import print_function
 import sys
 import numpy as np
 from numpy import ma
+from numpy.lib.stride_tricks import as_strided
 import warnings
 import getopt
 import os
@@ -86,7 +87,7 @@ least_significant_digit=1, bits will be 4.
         return datout
 
 def _StartCountStride(elem, shape, dimensions=None, grp=None, datashape=None,\
-        put=False, no_get_vars = True):
+        put=False, use_get_vars = False):
     """Return start, count, stride and indices needed to store/extract data
     into/from a netCDF variable.
 
@@ -139,7 +140,7 @@ def _StartCountStride(elem, shape, dimensions=None, grp=None, datashape=None,\
     grp  : netCDF Group
       The netCDF group to which the variable being set belongs to.
     datashape : sequence
-      The shape of the data that is being stored. Only needed by __setitime__
+      The shape of the data that is being stored. Only needed by __setitem__
     put : True|False (default False).  If called from __setitem__, put is True.
 
     Returns
@@ -178,6 +179,22 @@ def _StartCountStride(elem, shape, dimensions=None, grp=None, datashape=None,\
         nDims = 1
         shape = (1,)
 
+    # is there an unlimited dimension? (only defined for __setitem__)
+    if put:
+        hasunlim = False
+        unlimd={}
+        if dimensions:
+            for i in range(nDims):
+                dimname = dimensions[i]
+                # is this dimension unlimited?
+                # look in current group, and parents for dim.
+                dim = _find_dim(grp, dimname)
+                unlimd[dimname]=dim.isunlimited()
+                if unlimd[dimname]:
+                    hasunlim = True
+    else:
+        hasunlim = False
+
     # When a single array or (non-tuple) sequence of integers is given
     # as a slice, assume it applies to the first dimension,
     # and use ellipsis for remaining dimensions.
@@ -189,13 +206,14 @@ def _StartCountStride(elem, shape, dimensions=None, grp=None, datashape=None,\
                 elem.append(slice(None,None,None))
     else:   # Convert single index to sequence
         elem = [elem]
-    
-    hasEllipsis=False
-    for e in elem:
-        if type(e)==type(Ellipsis):
-            if hasEllipsis:
-                raise IndexError("At most one ellipsis allowed in a slicing expression")
-            hasEllipsis=True
+
+    # ensure there is at most 1 ellipse
+    #  we cannot use elem.count(Ellipsis), as with fancy indexing would occur
+    #  np.array() == Ellipsis which gives ValueError: The truth value of an
+    #  array with more than one element is ambiguous. Use a.any() or a.all()
+    if sum(1 for e in elem if e is Ellipsis) > 1:
+        raise IndexError("At most one ellipsis allowed in a slicing expression")
+
     # replace boolean arrays with sequences of integers.
     newElem = []
     IndexErrorMsg=\
@@ -216,13 +234,10 @@ def _StartCountStride(elem, shape, dimensions=None, grp=None, datashape=None,\
             raise IndexError("Index cannot be multidimensional")
         # set unlim to True if dimension is unlimited and put==True
         # (called from __setitem__)
-        if put and (dimensions is not None and grp is not None) and len(dimensions):
+        if hasunlim and put and dimensions:
             try:
                 dimname = dimensions[i]
-                # is this dimension unlimited?
-                # look in current group, and parents for dim.
-                dim = _find_dim(grp, dimname)
-                unlim = dim.isunlimited()
+                unlim = unlimd[dimname]
             except IndexError: # more slices than dimensions (issue 371)
                 unlim = False
         else:
@@ -256,7 +271,7 @@ Boolean array must have the same shape as the data along this dimension."""
             newElem.append(e)
         # slice or ellipsis object
         elif type(e) == slice or type(e) == type(Ellipsis):
-            if no_get_vars and type(e) == slice and e.step not in [None,-1,1] and\
+            if not use_get_vars and type(e) == slice and e.step not in [None,-1,1] and\
                dimensions is not None and grp is not None:
                 # convert strided slice to integer sequence if possible
                 # (this will avoid nc_get_vars, which is slow - issue #680).
@@ -281,7 +296,7 @@ Boolean array must have the same shape as the data along this dimension."""
                 newElem.append(e)
             except:
                 raise IndexError(IndexErrorMsg)
-        if type(e)==type(Ellipsis): 
+        if type(e)==type(Ellipsis):
             i+=1+nDims-len(elem)
         else:
             i+=1
@@ -304,10 +319,10 @@ Boolean array must have the same shape as the data along this dimension."""
                 ee = range(start,stop,step)
             except ValueError: # start, stop or step is not valid for a range
                 ee = False
-            if no_get_vars and ee and len(e) == len(ee) and (e == np.arange(start,stop,step)).all():
+            if ee and len(e) == len(ee) and (e == np.arange(start,stop,step)).all():
                 # don't convert to slice unless abs(stride) == 1
                 # (nc_get_vars is very slow, issue #680)
-                if step not in [1,-1]:
+                if not use_get_vars and step not in [1,-1]:
                     newElem.append(e)
                 else:
                     newElem.append(slice(start,stop,step))
@@ -341,6 +356,27 @@ Boolean array must have the same shape as the data along this dimension."""
         else:
             sdim.append(1)
 
+    # broadcast data shape when assigned to full variable (issue #919)
+    try:
+        fullslice = elem.count(slice(None,None,None)) == len(elem)
+    except: # fails if elem contains a numpy array.
+        fullslice = False
+    if fullslice and datashape and put and not hasunlim:
+        datashape = broadcasted_shape(shape, datashape)
+
+    # pad datashape with zeros for dimensions not being sliced (issue #906)
+    # only used when data covers slice over subset of dimensions
+    if datashape and len(datashape) != len(elem) and\
+       len(datashape) == sum(1 for e in elem if type(e) == slice):
+        datashapenew = (); i=0
+        for e in elem:
+            if type(e) != slice and not np.iterable(e): # scalar integer slice
+                datashapenew = datashapenew + (0,)
+            else: # slice object
+                datashapenew = datashapenew + (datashape[i],)
+                i+=1
+        datashape = datashapenew
+
     # Create the start, count, stride and indices arrays.
 
     sdim.append(max(nDims, 1))
@@ -355,12 +391,9 @@ Boolean array must have the same shape as the data along this dimension."""
 
         # set unlim to True if dimension is unlimited and put==True
         # (called from __setitem__). Note: grp and dimensions must be set.
-        if put and (dimensions is not None and grp is not None) and len(dimensions):
+        if hasunlim and put and dimensions:
             dimname = dimensions[i]
-            # is this dimension unlimited?
-            # look in current group, and parents for dim.
-            dim = _find_dim(grp, dimname)
-            unlim = dim.isunlimited()
+            unlim = unlimd[dimname]
         else:
             unlim = False
 
@@ -376,10 +409,13 @@ Boolean array must have the same shape as the data along this dimension."""
             if unlim and e.stop is not None and e.stop > shape[i]:
                 length = e.stop
             elif unlim and e.stop is None and datashape != ():
-                if e.start is None:
-                    length = datashape[i]
-                else:
-                    length = e.start+datashape[i]
+                try:
+                    if e.start is None:
+                        length = datashape[i]
+                    else:
+                        length = e.start+datashape[i]
+                except IndexError:
+                    raise IndexError("shape of data does not conform to slice")
             else:
                 if unlim and datashape == () and len(dim) == 0:
                     # writing scalar along unlimited dimension using slicing
@@ -472,9 +508,9 @@ def ncinfo():
     usage = """
  Print summary information about a netCDF file.
 
- usage: %s [-h] [-g grp or --group=grp] [-v var or --variable=var] [-d dim or --dimension=dim] filename
+ usage: %s [-h/--help] [-g grp or --group=grp] [-v var or --variable=var] [-d dim or --dimension=dim] filename
 
- -h -- Print usage message.
+ -h/--help -- Print usage message.
  -g <group name> or --group=<group name> -- Print info for this group
       (default is root group). Nested groups specified
       using posix paths ("group1/group2/group3").
@@ -498,7 +534,7 @@ def ncinfo():
     # Get the options
     group = None; var = None; dim=None
     for option in opts:
-        if option[0] == '-h':
+        if option[0] == '-h' or option[0] == '--help':
             sys.stderr.write(usage)
             sys.exit(0)
         elif option[0] == '--group' or option[0] == '-g':
@@ -513,7 +549,12 @@ def ncinfo():
             sys.exit(0)
 
     # filename passed as last argumenbt
-    filename = pargs[-1]
+    try:
+        filename = pargs[-1]
+    except IndexError:
+        sys.stdout.write("You need to pass netcdf filename!\n.")
+        sys.stderr.write(usage)
+        sys.exit(0)
 
     f = Dataset(filename)
     if group is None:
@@ -526,9 +567,9 @@ def ncinfo():
                 print(f.dimensions[dim])
     else:
         if var is None and dim is None:
-            print(getgrp(f,group))
+            print(_getgrp(f,group))
         else:
-            g = getgrp(f,group)
+            g = _getgrp(f,group)
             if var is not None:
                 print(g.variables[var])
             if dim is not None:
@@ -608,8 +649,8 @@ def nc4tonc3():
     usage = """
  Convert a netCDF 4 file (in NETCDF4_CLASSIC format) to netCDF 3 format.
 
- usage: %s [-h] [-o] [--chunk] netcdf4filename netcdf3filename
- -h -- Print usage message.
+ usage: %s [-h/--help] [-o] [--chunk] netcdf4filename netcdf3filename
+ -h/--help -- Print usage message.
  -o -- Overwrite destination file (default is to raise an error if output file already exists).
  --quiet=(0|1)  -- if 1, don't print diagnostic information.
  --format -- netcdf3 format to use (NETCDF3_64BIT by default, can be set to NETCDF3_CLASSIC)
@@ -635,7 +676,7 @@ def nc4tonc3():
 
     # Get the options
     for option in opts:
-        if option[0] == '-h':
+        if option[0] == '-h' or option[0] == '--help':
             sys.stderr.write(usage)
             sys.exit(0)
         elif option[0] == '-o':
@@ -743,11 +784,18 @@ def _nc3tonc4(filename3,filename4,unpackshort=True,
         if dounpackshort:
             if not quiet: sys.stdout.write('unpacking short integers to floats ...\n')
             sys.stdout.write('')
+        # is there missing value?
         if hasattr(ncvar, '_FillValue'):
-            FillValue = ncvar._FillValue
+            fillvalue3 = ncvar._FillValue
+        elif hasattr(ncvar, 'missing_value'):
+            fillvalue3 = ncvar.missing_value
         else:
-            FillValue = None
-        var = ncfile4.createVariable(varname,datatype,ncvar.dimensions, fill_value=FillValue, least_significant_digit=lsd,zlib=zlib,complevel=complevel,shuffle=shuffle,fletcher32=fletcher32)
+            fillvalue3 = None
+        if fillvalue3 is not None:
+            fillvalue4 = fillvalue3 if not dounpackshort else mval
+        else:
+            fillvalue4 = None
+        var = ncfile4.createVariable(varname,datatype,ncvar.dimensions, fill_value=fillvalue4, least_significant_digit=lsd,zlib=zlib,complevel=complevel,shuffle=shuffle,fletcher32=fletcher32)
         # fill variable attributes.
         attdict = ncvar.__dict__
         if '_FillValue' in attdict: del attdict['_FillValue']
@@ -756,15 +804,8 @@ def _nc3tonc4(filename3,filename4,unpackshort=True,
         if dounpackshort and 'scale_factor' in attdict:
             del attdict['scale_factor']
         if dounpackshort and 'missing_value' in attdict:
-            attdict['missing_value']=mval
+            attdict['missing_value'] = fillvalue4
         var.setncatts(attdict)
-        #for attname in ncvar.ncattrs():
-        #    if attname == '_FillValue': continue
-        #    if dounpackshort and attname in ['add_offset','scale_factor']: continue
-        #    if dounpackshort and attname == 'missing_value':
-        #        setattr(var,attname,mval)
-        #    else:
-        #        setattr(var,attname,getattr(ncvar,attname))
         # fill variables with data.
         if hasunlimdim: # has an unlim dim, loop over unlim dim index.
             # range to copy
@@ -774,32 +815,11 @@ def _nc3tonc4(filename3,filename4,unpackshort=True,
                 for n in range(start, stop, step):
                     nmax = n+nchunk
                     if nmax > istop: nmax=istop
-                    idata = ncvar[n:nmax]
-                    if dounpackshort:
-                        tmpdata = (ncvar.scale_factor*idata.astype('f')+ncvar.add_offset).astype('f')
-                        if hasattr(ncvar,'missing_value'):
-                            tmpdata = np.where(idata == ncvar.missing_value, mval, tmpdata)
-                    else:
-                        tmpdata = idata
-                    var[n-istart:nmax-istart] = tmpdata
+                    var[n-istart:nmax-istart] = ncvar[n:nmax]
             else:
-                idata = ncvar[:]
-                if dounpackshort:
-                    tmpdata = (ncvar.scale_factor*idata.astype('f')+ncvar.add_offset).astype('f')
-                    if hasattr(ncvar,'missing_value'):
-                        tmpdata = np.where(idata == ncvar.missing_value, mval, tmpdata)
-                else:
-                    tmpdata = idata
-                var[0:len(unlimdim)] = tmpdata
+                var[0:len(unlimdim)] = ncvar[:]
         else: # no unlim dim or 1-d variable, just copy all data at once.
-            idata = ncvar[:]
-            if dounpackshort:
-                tmpdata = (ncvar.scale_factor*idata.astype('f')+ncvar.add_offset).astype('f')
-                if hasattr(ncvar,'missing_value'):
-                    tmpdata = np.where(idata == ncvar.missing_value, mval, tmpdata)
-            else:
-                tmpdata = idata
-            var[:] = tmpdata
+            var[:] = ncvar[:]
         ncfile4.sync() # flush data to disk
     # close files.
     ncfile3.close()
@@ -813,8 +833,8 @@ def nc3tonc4():
  to floats, and adding zlib compression (with the HDF5 shuffle filter and fletcher32 checksum).
  Data may also be quantized (truncated) to a specified precision to improve compression.
 
- usage: %s [-h] [-o] [--vars=var1,var2,..] [--zlib=(0|1)] [--complevel=(1-9)] [--shuffle=(0|1)] [--fletcher32=(0|1)] [--unpackshort=(0|1)] [--quantize=var1=n1,var2=n2,..] netcdf3filename netcdf4filename
- -h -- Print usage message.
+ usage: %s [-h/--help] [-o] [--vars=var1,var2,..] [--zlib=(0|1)] [--complevel=(1-9)] [--shuffle=(0|1)] [--fletcher32=(0|1)] [--unpackshort=(0|1)] [--quantize=var1=n1,var2=n2,..] netcdf3filename netcdf4filename
+ -h/--help -- Print usage message.
  -o -- Overwrite destination file (default is to raise an error if output file already exists).
  --vars -- comma separated list of variable names to copy (default is to copy
     all variables)
@@ -879,7 +899,7 @@ def nc3tonc4():
 
     # Get the options
     for option in opts:
-        if option[0] == '-h':
+        if option[0] == '-h' or option[0] == '--help':
             sys.stderr.write(usage)
             sys.exit(0)
         elif option[0] == '-o':
@@ -942,3 +962,12 @@ def nc3tonc4():
         fletcher32=fletcher32,clobber=overwritefile,lsd_dict=lsd_dict,
         nchunk=chunk,quiet=quiet,vars=vars,classic=classic,
         istart=istart,istop=istop)
+
+def broadcasted_shape(shp1, shp2):
+    # determine shape of array of shp1 and shp2 broadcast against one another.
+    x = np.array([1])
+    # trick to define array with certain shape that doesn't allocate all the
+    # memory.
+    a = as_strided(x, shape=shp1, strides=[0] * len(shp1))
+    b = as_strided(x, shape=shp2, strides=[0] * len(shp2))
+    return np.broadcast(a, b).shape
