@@ -1,6 +1,7 @@
 import glob, os, sys, unittest, struct
 from netCDF4 import getlibversion,__hdf5libversion__,__netcdf4libversion__,__version__
-from netCDF4 import __has_cdf5_format__, __has_nc_inq_path__, __has_nc_par__
+from netCDF4 import __has_cdf5_format__, __has_nc_inq_path__, __has_nc_create_mem__, \
+                    __has_parallel4_support__, __has_pnetcdf_support__
 
 # can also just run
 # python -m unittest discover . 'tst*py'
@@ -15,12 +16,15 @@ if python3:
 else:
     test_files.remove('tst_unicode3.py')
     sys.stdout.write('not running tst_unicode3.py ...\n')
-if __netcdf4libversion__ < '4.2.1' or __has_nc_par__:
+if __netcdf4libversion__ < '4.2.1' or __has_parallel4_support__ or __has_pnetcdf_support__:
     test_files.remove('tst_diskless.py')
     sys.stdout.write('not running tst_diskless.py ...\n')
 if not __has_nc_inq_path__:
     test_files.remove('tst_filepath.py')
     sys.stdout.write('not running tst_filepath.py ...\n')
+if not __has_nc_create_mem__:
+    test_files.remove('tst_create_mem.py')
+    sys.stdout.write('not running tst_create_mem.py ...\n')
 if not __has_cdf5_format__ or struct.calcsize("P") < 8:
     test_files.remove('tst_cdf5.py')
     sys.stdout.write('not running tst_cdf5.py ...\n')
@@ -29,6 +33,10 @@ if not __has_cdf5_format__ or struct.calcsize("P") < 8:
 if os.getenv('NO_NET'):
     test_files.remove('tst_dap.py');
     sys.stdout.write('not running tst_dap.py ...\n')
+else:
+    # run opendap test first (issue #856).
+    test_files.remove('tst_dap.py')
+    test_files.insert(0,'tst_dap.py')
 
 # Build the test suite from the tests found in the test files.
 testsuite = unittest.TestSuite()
