@@ -101,6 +101,13 @@ class DimensionsTestCase(unittest.TestCase):
         # check that dimension lengths are correct.
         for name,dim in g.dimensions.items():
             self.assertTrue(len(dim) == lensdict[name])
+        # check get_dims variable method
+        dim_tuple = vg.get_dims()
+        # some dimensions from parent group
+        dim_tup1 = (f.dimensions['level'],g.dimensions['lat'],\
+                    g.dimensions['lon'],f.dimensions['time'])
+        dim_tup2 = vg.get_dims()
+        assert(dim_tup1 == dim_tup2)
         # check that isunlimited() method works.
         for name,dim in g.dimensions.items():
             self.assertTrue(dim.isunlimited() == unlimdict[name])
